@@ -1,8 +1,8 @@
 /***************************************************************************
                              -------------------
-    cvs         : $Id: ktoblzcheck.cc,v 1.22 2003/09/01 09:31:49 cstim Exp $
+    cvs         : $Id: ktoblzcheck.cc,v 1.28 2004/09/10 08:25:27 cstim Exp $
     begin       : Sat Aug 10 2002
-    copyright   : (C) 2002 by Fabian Kaiser
+    copyright   : (C) 2002, 2003 by Fabian Kaiser and Christian Stimming
     email       : fabian@openhbci.de
 
  ***************************************************************************
@@ -24,7 +24,9 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include "ktoblzcheck.h"
 #include <fstream>
@@ -54,20 +56,27 @@ AccountNumberCheck::Record::Record(unsigned long id,
 {
 }
 
+
 AccountNumberCheck::AccountNumberCheck() 
+    : data() // std::map doesn't take size as argument
+{
+   string registry_path = accnum_getRegKey("datadir");
+   string data_path = BANKDATA_PATH;
+   string filename = (registry_path.empty() ? data_path : registry_path)
+      + 
+#if OS_WIN32
+      "\\"
+#else
+      "/"
+#endif
+      + std::string("bankdata.txt");
+   readFile(filename);
+}
+
+AccountNumberCheck::AccountNumberCheck(const string& filename) 
     : 
     data() // std::map doesn't take size as argument
 {
-  // Disabled COMPILE_RESOURCE because the big list cannot be handled by the compiler anyway.
-
-  string data_path = BANKDATA_PATH;
-  string filename = data_path + "/bankdata.txt";
-  readFile(filename);
-}
-
-AccountNumberCheck::AccountNumberCheck(const string& filename) {
-  // I guess it is okay to simply accept this function call,
-  // even if COMPILE_RESOURCE was activated.
   readFile(filename);
 }
 
@@ -77,7 +86,7 @@ AccountNumberCheck::~AccountNumberCheck()
 }
 
 
-#define LINEBUFFER_SIZE 200
+#define LINEBUFFER_SIZE 250
 void 
 AccountNumberCheck::readFile(const string &filename) 
 {
@@ -162,17 +171,20 @@ class MatchBlz {
 };
 
 const AccountNumberCheck::Record& 
-AccountNumberCheck::findBank(const string& bankId) const {  
-
+AccountNumberCheck::findBank(const string& bankId) const 
+{
   unsigned long lbankId = atol(bankId.c_str());
   banklist_type::const_iterator iter;
+
+  // Lookup the object
   iter = data.find(lbankId);
 
+  // Did we find it? Yes, return the reference.
   if (iter != data.end()) {
     return *(iter->second);
   }
-  
-  throw -1;
+  else
+    throw -1;
 }
 
 
@@ -197,7 +209,7 @@ AccountNumberCheck::check(const string& bankId, const string& accountId,
     try {
       Record rec = findBank(bankId);
       method = rec.method;
-    } catch (int i) {
+    } catch (int) {
 	// bank was not found, return error if not forced to use a spec. method
 	return BANK_NOT_KNOWN;
     }
@@ -396,7 +408,7 @@ AccountNumberCheck::check(const string& bankId, const string& accountId,
         // iterated transformation
             return algo07(account, transform);
         }
-        return ERROR;
+        //return ERROR;
   }
   if ("28" == method) {
 	number2Array("8765432000", weight);
@@ -572,7 +584,9 @@ AccountNumberCheck::check(const string& bankId, const string& accountId,
             number2Array("3987654320", weight);
             return algo01(11, weight, false, 10, account);
         }
-        return algo04(bankId, accountId);
+	// Hint by Erik Kerger <erik@kedo.com>, 2004-08-14: return
+	// algo04a instead of algo04
+        return algo04a(bankId, accountId);
   }
   if ("54" == method) {
 	// ids must start with 49
@@ -1028,7 +1042,13 @@ AccountNumberCheck::check(const string& bankId, const string& accountId,
 		return OK;
 	  else {	
 		number2Array("0987650432", weight); weight[0] = 10;	
-		return algo01(11, weight, false, 7, account);
+		tmp = algo01(11, weight, false, 7, account);
+		if (OK == tmp)
+		  return OK;
+		else {
+		  number2Array("9058420000", weight); weight[1] = 10;
+		  return algo01(11, weight, false, 7, account);
+		}
 	  }
 	}
   }
@@ -1088,7 +1108,7 @@ AccountNumberCheck::check(const string& bankId, const string& accountId,
   }
   if ("97" == method) {
 	string tmp = array2Number(account).substr(0, 9);
-	long long valueX = number2LongLong(tmp);
+	long_long valueX = number2LongLong(tmp);
 	valueX -= (valueX / 11) * 11;
 	valueX = valueX % 10;
 	
@@ -1226,6 +1246,14 @@ AccountNumberCheck::check(const string& bankId, const string& accountId,
         number2Array("0001212120", weight);
         return algo01(10,weight, true, 10, account);
   }
+  if ("A9" == method) {
+	number2Array("1731731730", weight);
+	if(algo01(10, weight, false, 10, account) != OK) {
+	  number2Array("4327654320", weight);
+	  return algo01(11, weight, false, 10, account);
+	}
+	return OK;
+  }
   std::cerr << "AccountNumberCheck::check: Specified method '" << method 
 	    << "' is unknown." << std::endl;
   return UNKNOWN;
